library(quantreg)

dr.test1 <- function(formula, dat, bsrep=19) {

 # get variable names
 outname <- all.vars(formula)[1]
 covname <- all.vars(formula)[-1]
 n <- length(dat[,outname])
   
 y  <- dat[,outname]
 x  <- model.matrix(formula, data=dat)[,-1]

 taus <- seq(1/n,1- 1/n,length=100)
 helpfun1 <- function(cdfobj,y,ygrid,...) {approx(x=ygrid,y=sort(cdfobj),xout=y,rule=2,method="linear")$y}

 helpfun2 <- function(cdfobj,ys,...) {approx(x=cdfobj,y=ys,xout=runif(1),rule=2,method="linear",ties=max)$y}

 ygrid <- quantile(y,taus)
 ygrid <- ygrid[!duplicated(ygrid)]
 
 cond.cdf1 <- matrix(1/n, ncol=n,nrow=length(ygrid))
 cond.cdf1[length(ygrid),] <- rep(1 - 1/n,n)

 for (i in 2:(length(ygrid)-1)){
    mod <- glm(I(y <= ygrid[i]) ~ x, family=binomial(link="probit"))
    cond.cdf1[i,] <- predict(mod, type="response")
  }

 startvals <- sort(sample(1:n,round(min(c(n, 300))),replace=FALSE))
 x.ev <- x[startvals,]
 y.ev <- y[startvals]

 cond.cdf <- apply(cond.cdf1,2, helpfun1,y=y.ev,ygrid=ygrid) 

 ## Calculate CDF difference for a number of starting values
 cdfdiff <- 0
  for (i in 1:length(y.ev)) {
    cdfdiff[i] <- mean(( cond.cdf[i,]  - I(y <= y.ev[i]))*apply(t(x)- x.ev[i,] <= 0, 2,prod))
  }

 teststat.ks <- sqrt(n)*max(abs(cdfdiff))
 teststat.cvm <- n*sum(cdfdiff^2)
 
 ## Start the bootstrap procedure
 
 ## Initiate vector of bootstrap replications of test statistic
 teststat.ks.bs <- 0
 teststat.cvm.bs <- 0
 
 ## Loop over bootstrap repetitions
 for (j in 1:bsrep) {

  ##generate bootstrap data
  bsamp <- sample(1:n,n,replace=TRUE)
  dat.bs <- model.frame(formula,data=dat[bsamp,], drop.unused.levels = TRUE) 
  dat.bs[,outname] <- apply(cond.cdf1[,bsamp],2,helpfun2,ys=ygrid)
   
  y.bs  <- dat.bs[,outname]
  x.bs <- model.matrix(formula, data=dat.bs)[,-1]
  
  ygrid.bs <- quantile(y.bs,taus,type=1)
  ygrid.bs <- ygrid.bs[!duplicated(ygrid.bs)]
  
  cond.cdf.bs <- matrix(1/n,ncol=n,nrow=length(ygrid.bs))
  cond.cdf.bs[length(ygrid.bs),] <- rep(1 - 1/n,n)
  
  for (i in 2:(length(ygrid.bs)-1)){
    mod <- glm(I(y.bs <= ygrid.bs[i]) ~ x.bs, family=binomial(link="probit"))
    cond.cdf.bs[i,] <- predict(mod, type="response")
  }
  
  startvals <- sort(sample(1:n,round(min(c(n, 300))),replace=FALSE))
  x.ev.bs <- x.bs[startvals,] 
  y.ev.bs <- y.bs[startvals] 
  
 cond.cdf.bs <- apply(cond.cdf.bs,2, helpfun1,y=y.ev.bs,ygrid=ygrid.bs)
 
 ## Calculate CDF difference for a number of starting values
  cdfdiff.bs <- 0
  for (i in 1:length(y.ev.bs)) {
  cdfdiff.bs[i] <- mean(( cond.cdf.bs[i,]  - I(y.bs <= y.ev.bs[i]))*
    apply(t(x.bs)- x.ev.bs[i,] <= 0, 2,prod))
  }

 teststat.ks.bs[j] <- sqrt(n)*max(abs(cdfdiff.bs))
 teststat.cvm.bs[j] <- n*sum((cdfdiff.bs)^2)

}

pval.ks <- mean(teststat.ks.bs > teststat.ks, na.rm=TRUE)
pval.cvm <- mean(teststat.cvm.bs > teststat.cvm, na.rm=TRUE)


list(teststat.ks=teststat.ks, pval.ks=pval.ks,teststat.cvm=teststat.cvm, pval.cvm=pval.cvm, test="Distributional Regression")

}


