library(quantreg)

locscale.test2 <- function(formula, dat, scaletest=TRUE, bsrep=19) {

 # get variable names
 outname <- all.vars(formula)[1]
 covname <- all.vars(formula)[-1]
 n <- length(dat[,outname])
 
 helpfun1 <- function(quantobj,yval,taugrid,...) {
  approx(y=taugrid,x=sort(quantobj),xout=yval,rule=2,method="linear",ties=max)$y}
 
 helpfun2 <- function(mv,res, taugrid) {mv[1] + mv[2]*quantile(res,taugrid,type=1)}
 
 helpfun3 <- function(quantobj,taugrid,...) { 
  approx(x=taugrid, y=sort(quantobj),xout=runif(1),rule=2,method="linear")$y} 
  
 taus <-  seq(1/n,1-1/n,length=75 + n/8)
 mod <- lm(formula, data=dat)
 y  <- dat[,outname]
 x  <- model.matrix(mod)[,-1]
 res <- resid(mod)
 mm <- predict(mod)
 
 if (scaletest) {              
   para <- optim(par = c(sd(res),rep(0,length(coef(mod))-1)),
                fn = function(theta){
                    vv <- model.matrix(mod)%*%theta
                    sum( (res^2 - vv^2)^2 )})$par
  vv <- abs(model.matrix(formula,data=dat)%*%para)
 } else { 
  vv <- rep(sd(res),n)
 }
 
 res <- res/vv

 startvals <- sort(sample(1:n,round(min(c(n, 300))),replace=FALSE))
 x.ev <- x[startvals,] 
 y.ev <- y[startvals]

 cond.quant<- t(apply( cbind(mm,vv),1, helpfun2, res = res, taugrid = taus ))
 cond.cdf <- apply(cond.quant,1, helpfun1,yval=y.ev,taugrid=taus)
 
 ## Calculate CDF difference for a number of starting values
 cdfdiff <- 0
  for (i in 1:length(y.ev)) {
    cdfdiff[i] <- mean(( cond.cdf[i,]  - I(y <= y.ev[i]))*apply(t(x)- x.ev[i,] <= 0, 2,prod))
  }

 teststat.ks <- max(abs(cdfdiff))
 teststat.cvm <- sum(cdfdiff^2)
 
 ## Start the bootstrap procedure
 
 ## Initiate vector of bootstrap replications of test statistic
 teststat.ks.bs <- 0
 teststat.cvm.bs <- 0
 
 ## Loop over bootstrap repetitions
 for (j in 1:bsrep) {

 ##generate bootstrap data
  bsamp <- sample(1:n,n,replace=TRUE)
  dat.bs <- model.frame(formula,data=dat[bsamp,], drop.unused.levels = TRUE) 
  dat.bs[,outname] <- apply(cond.quant[bsamp,],1,helpfun3,taugrid=taus)
  
   y.bs <- dat.bs[,outname]
   x.bs <- model.matrix(formula,data=dat.bs)[,-1]
   mod <- lm(formula, data = dat.bs)
   res <- resid(mod)
   mm <- predict(mod)
 
 if (scaletest) {              
   para <- optim(par = c(sd(res),rep(0,length(coef(mod))-1)),
                fn = function(theta){
                    vv <- model.matrix(formula,data=dat.bs)%*%theta
                    sum( (res^2 - vv^2)^2 )})$par
  vv <- abs(model.matrix(formula,data=dat.bs)%*%para)
 } else { 
  vv <- rep(sd(res),n)
 }
 
 res <- res/vv

 startvals <- sort(sample(1:n,round(min(c(n, 300))),replace=FALSE))
 x.ev <- x.bs[startvals,] 
 y.ev <- y.bs[startvals]

 cond.quant.bs<- t(apply( cbind(mm,vv),1, helpfun2, res = res, taugrid = taus))
 cond.cdf.bs <- apply(cond.quant.bs,1, helpfun1,yval=y.ev,taugrid=taus)

 ## Calculate CDF difference for a number of starting values
 cdfdiff.bs <- 0
  for (i in 1:length(y.ev)) {
  cdfdiff.bs[i] <- mean((cond.cdf.bs[i,] - I(y.bs <= y.ev[i]))*apply(t(x.bs)- x.ev[i,] <= 0, 2,prod) )
  }
  
 teststat.ks.bs[j] <- max(abs(cdfdiff.bs))
 teststat.cvm.bs[j] <- sum((cdfdiff.bs)^2)
}

pval.ks <- mean(teststat.ks.bs > teststat.ks, na.rm=TRUE)
pval.cvm <- mean(teststat.cvm.bs > teststat.cvm, na.rm=TRUE)

list(teststat.ks=teststat.ks, pval.ks=pval.ks,teststat.cvm=teststat.cvm, pval.cvm=pval.cvm, test="LSS TEST")

}


