library(quantreg)

qr.test1 <- function(formula, dat, bsrep=19) {

  # get variable names
 outname <- all.vars(formula)[1]
 covname <- all.vars(formula)[-1]
 n <- length(dat[,outname])

 taus <-  seq(1/n,1-1/n,length=100)
 mod.rq <- rq(formula, data=dat, tau=taus)
 y  <- dat[,outname]
 x  <- model.matrix(mod.rq, data=dat)[,-1]
 
 startvals <- sort(sample(1:n,round(min(c(n, 300))),replace=FALSE))
 x.ev <- x[startvals,]
 y.ev <- y[startvals]
 
  
 helpfun1 <- function(quantobj,yval,taugrid,...) {
   approx(y=taugrid,x=sort(quantobj),xout=yval,rule=2,method="linear",ties=max)$y}
   
 helpfun2 <- function(quantobj,taugrid,...) { 
  approx(x=taugrid, y=sort(quantobj),xout=runif(1),rule=2,method="linear")$y} 

 cond.quant <- predict(mod.rq) 
 cond.cdf <- apply(cond.quant,1, helpfun1,yval=y.ev,taugrid=taus)

 ## Calculate CDF difference for a number of starting values
 cdfdiff <- 0
  for (i in 1:length(y.ev)) {
    cdfdiff[i] <- mean(( cond.cdf[i,] - I(y <= y.ev[i]))*apply(t(x)- x.ev[i,] <= 0, 2,prod))
  }

 teststat.ks <- sqrt(n)*max(abs(cdfdiff))
 teststat.cvm <- sum(cdfdiff^2)
 
 ## Start the bootstrap procedure
 
 ## Initiate vector of bootstrap replications of test statistic
 teststat.ks.bs <- 0
 teststat.cvm.bs <- 0
 
 ## Loop over bootstrap repetitions
 for (j in 1:bsrep) {

 ##generate bootstrap data
  bsamp <- sample(1:n,n,replace=TRUE)
  dat.bs <- model.frame(formula,data=dat[bsamp,], drop.unused.levels = TRUE) 
  dat.bs[,outname] <- apply(cond.quant[bsamp,],1,helpfun2,taugrid=taus)

 mod.rq <- rq(formula, data=dat.bs, tau=taus)
 y.bs  <- dat.bs[,outname]
 x.bs  <- model.matrix(mod.rq, data=dat.bs)[,-1]
 
 startvals <- sort(sample(1:n,round(min(c(n, 300))),replace=FALSE))
 x.ev <- x.bs[startvals,] 
 y.ev <- y.bs[startvals]

 ## Calculate QR
 cond.quant.bs <- predict(mod.rq) 
 cond.cdf <- apply(cond.quant.bs,1, helpfun1,yval=y.ev,taugrid=taus)

 ## Calculate CDF difference for a number of starting values
 cdfdiff.bs <- 0
  for (i in 1:length(y.ev)) {
    cdfdiff.bs[i] <- mean((cond.cdf[i,]- I(y.bs <= y.ev[i]))*apply(t(x.bs)- x.ev[i,] <= 0, 2,prod))
  }

 teststat.ks.bs[j] <- sqrt(n)*max(abs(cdfdiff.bs))
 teststat.cvm.bs[j] <- sum((cdfdiff.bs)^2)

}

pval.ks <- mean(teststat.ks.bs > teststat.ks, na.rm=TRUE)
pval.cvm <- mean(teststat.cvm.bs > teststat.cvm, na.rm=TRUE)


list(teststat.ks=teststat.ks, pval.ks=pval.ks,teststat.cvm=teststat.cvm, pval.cvm=pval.cvm, test="Linear Quantile Regression")

}


